import os
import shutil
from tkinter import *
from tkinter import ttk, filedialog, messagebox
import ttkbootstrap as tb

class StoreBuilderApp:
    def __init__(self, root_window):
        """
        פונקציית הבנאי הראשית של האפליקציה.
        מאתחלת את החלון הראשי, מבני הנתונים, ואת כל רכיבי הממשק.
        """
        self.root = root_window
        self.root.title("בנאי תיקיות לחנות האפליקציות")
        self.root.geometry("950x700")

        # מבנה הנתונים הראשי שמחזיק את כל המידע על החנות
        # פורמט: { "שם_תיקייה_קטגוריה": {"display_name": "שם קטגוריה לתצוגה", "apks": ["/path/to/app1.apk", ...]}, ... }
        self.store_data = {}
        
        # מילון עזר למצוא את שם התיקייה לפי השם המוצג ברשימה
        self.display_to_folder_map = {}
        
        # משתנה שיחזיק את שם התיקייה של הקטגוריה שנבחרה כרגע
        self.current_category_folder = None

        self.setup_ui()

    def setup_ui(self):
        """
        בונה ומסדר את כל הרכיבים הגרפיים של הממשק.
        """
        # --- מסגרת ראשית ---
        main_frame = ttk.Frame(self.root, padding="15")
        main_frame.pack(fill=BOTH, expand=YES)

        # --- שלב 1: ניהול קטגוריות ---
        cat_frame = ttk.LabelFrame(main_frame, text="שלב 1: ניהול קטגוריות", padding="10")
        cat_frame.pack(fill=X, pady=(0, 15))
        
        add_cat_subframe = ttk.Frame(cat_frame)
        add_cat_subframe.pack(fill=X, pady=(0,10))
        add_cat_subframe.columnconfigure(0, weight=1)
        
        self.category_name_entry = ttk.Entry(add_cat_subframe, font=("Helvetica", 12), justify=RIGHT)
        self.category_name_entry.grid(row=0, column=0, sticky="ew", padx=(5,0))
        self.category_name_entry.bind("<Return>", lambda event: self.add_category())

        add_cat_btn = ttk.Button(add_cat_subframe, text="הוסף קטגוריה", command=self.add_category, style="success.TButton")
        add_cat_btn.grid(row=0, column=1, padx=(5,0))
        
        remove_cat_btn = ttk.Button(add_cat_subframe, text="הסר קטגוריה", command=self.remove_category, style="danger.TButton")
        remove_cat_btn.grid(row=0, column=2, padx=(5,0))
        
        self.category_listbox = Listbox(cat_frame, font=("Helvetica", 12), justify=RIGHT, height=6)
        self.category_listbox.pack(fill=X)
        self.category_listbox.bind("<<ListboxSelect>>", self.on_category_select)

        # --- שלב 2: ניהול אפליקציות ---
        self.apk_frame = ttk.LabelFrame(main_frame, text="שלב 2: ניהול אפליקציות (בחר קטגוריה)", padding="10")
        self.apk_frame.pack(fill=BOTH, expand=YES, pady=(0, 15))
        self.apk_frame.rowconfigure(0, weight=1)
        self.apk_frame.columnconfigure(0, weight=1)
        
        self.apk_listbox = Listbox(self.apk_frame, font=("Helvetica", 11))
        self.apk_listbox.grid(row=0, column=0, sticky="nsew", columnspan=2)

        apk_buttons_frame = ttk.Frame(self.apk_frame)
        apk_buttons_frame.grid(row=1, column=0, sticky="ew", columnspan=2, pady=(10,0))
        apk_buttons_frame.columnconfigure(0, weight=1)
        apk_buttons_frame.columnconfigure(1, weight=1)
        
        self.add_apk_btn = ttk.Button(apk_buttons_frame, text="הוסף קבצי APK...", command=self.add_apks, style="info.TButton")
        self.add_apk_btn.grid(row=0, column=1, sticky="ew", padx=(5,0))
        self.remove_apk_btn = ttk.Button(apk_buttons_frame, text="הסר APK נבחר", command=self.remove_app, style="warning.TButton")
        self.remove_apk_btn.grid(row=0, column=0, sticky="ew")

        # --- שלב 3: יצירת המבנה הסופי ---
        build_frame = ttk.LabelFrame(self.root, text="שלב 3: יצירת מבנה התיקיות", padding="10")
        build_frame.pack(fill=X, side=BOTTOM, padx=10, pady=(0, 10))
        
        self.build_button = ttk.Button(build_frame, text="צור תיקיית 'store'", command=self.build_structure, style="primary.TButton")
        self.build_button.pack(expand=YES, fill=X, ipady=10)

        self.status_label = ttk.Label(build_frame, text="מוכן. אנא הוסף קטגוריה כדי להתחיל.", justify=RIGHT, anchor=E)
        self.status_label.pack(fill=X, pady=5)
        
        # Disable APK section initially
        self.set_apk_section_state(DISABLED)

    def set_apk_section_state(self, state):
        """ מפעיל או משבית את כל הרכיבים באזור ניהול האפליקציות. """
        self.add_apk_btn.config(state=state)
        self.remove_apk_btn.config(state=state)
        self.apk_listbox.config(state=state)

    def add_category(self):
        """ מוסיף קטגוריה חדשה למבנה הנתונים ומרענן את התצוגה. """
        display_name = self.category_name_entry.get().strip()
        if not display_name:
            messagebox.showerror("שגיאה", "שם הקטגוריה לא יכול להיות ריק.")
            return
        
        folder_name = display_name.lower().replace(" ", "_")
        
        if folder_name in self.store_data:
            messagebox.showerror("שגיאה", f"קטגוריה עם שם התיקייה '{folder_name}' כבר קיימת.")
            return
        
        self.store_data[folder_name] = {"display_name": display_name, "apks": []}
        self.display_to_folder_map[display_name] = folder_name
        
        self.category_name_entry.delete(0, END)
        self.update_category_view()
        self.status_label.config(text=f"הקטגוריה '{display_name}' נוספה בהצלחה.")
        
    def remove_category(self):
        """ מסיר את הקטגוריה הנבחרת לאחר קבלת אישור. """
        if not self.category_listbox.curselection():
            messagebox.showwarning("אזהרה", "יש לבחור קטגוריה למחיקה.")
            return
            
        selected_display_name = self.category_listbox.get(self.category_listbox.curselection())
        folder_name = self.display_to_folder_map[selected_display_name]
        
        if messagebox.askyesno("אישור מחיקה", f"האם אתה בטוח שברצונך למחוק את הקטגוריה '{selected_display_name}' ואת כל האפליקציות שבה?"):
            del self.store_data[folder_name]
            del self.display_to_folder_map[selected_display_name]

            if self.current_category_folder == folder_name:
                self.current_category_folder = None
            
            self.update_category_view()
            self.status_label.config(text=f"הקטגוריה '{selected_display_name}' נמחקה.")

    def on_category_select(self, event):
        """ מופעל בעת בחירת קטגוריה. מציג את האפליקציות שלה ומפעיל את אזור שלב 2. """
        if not self.category_listbox.curselection():
            return
            
        selected_display_name = self.category_listbox.get(self.category_listbox.curselection())
        self.current_category_folder = self.display_to_folder_map[selected_display_name]
        
        self.apk_frame.config(text=f"שלב 2: ניהול אפליקציות עבור '{selected_display_name}'")
        self.set_apk_section_state(NORMAL)
        self.update_app_view()

    def update_category_view(self):
        """ מרענן את רשימת הקטגוריות בתצוגה. """
        self.category_listbox.delete(0, END)
        for display_name in sorted(self.display_to_folder_map.keys()):
            self.category_listbox.insert(END, display_name)
        
        if not self.current_category_folder or self.current_category_folder not in self.store_data:
            self.apk_frame.config(text="שלב 2: ניהול אפליקציות (בחר קטגוריה)")
            self.set_apk_section_state(DISABLED)
            self.update_app_view() # Clear the list

    def add_apks(self):
        """ פותח חלון לבחירת קבצי APK ומוסיף אותם לקטגוריה הנוכחית. """
        if not self.current_category_folder:
            messagebox.showwarning("אזהרה", "יש לבחור קטגוריה תחילה.")
            return

        file_paths = filedialog.askopenfilenames(
            title="בחר קבצי APK",
            filetypes=(("קבצי APK", "*.apk"), ("כל הקבצים", "*.*"))
        )
        if file_paths:
            count = 0
            current_apks = self.store_data[self.current_category_folder]["apks"]
            for path in file_paths:
                if path not in current_apks:
                    current_apks.append(path)
                    count += 1
            
            if count > 0:
                self.update_app_view()
                self.status_label.config(text=f"נוספו {count} קבצי APK חדשים.")
            else:
                self.status_label.config(text="לא נוספו קבצים חדשים (ייתכן שכבר קיימים).")
            
    def remove_app(self):
        """ מסיר את ה-APK הנבחר מהקטגוריה הנוכחית. """
        if not self.current_category_folder or not self.apk_listbox.curselection():
            messagebox.showwarning("אזהרה", "יש לבחור קובץ APK למחיקה.")
            return
            
        selected_index = self.apk_listbox.curselection()[0]
        apk_path_to_remove = self.store_data[self.current_category_folder]["apks"].pop(selected_index)
        self.update_app_view()
        self.status_label.config(text=f"הקובץ {os.path.basename(apk_path_to_remove)} הוסר.")

    def update_app_view(self):
        """ מרענן את רשימת קבצי ה-APK בתצוגה. """
        self.apk_listbox.delete(0, END)
        if self.current_category_folder and self.current_category_folder in self.store_data:
            apk_paths = self.store_data[self.current_category_folder]["apks"]
            if apk_paths:
                for path in apk_paths:
                    self.apk_listbox.insert(END, os.path.basename(path))
            else:
                self.apk_listbox.insert(END, "לא נוספו אפליקציות. לחץ על 'הוסף קבצי APK...'")
        
    def build_structure(self):
        """
        הפונקציה המרכזית שיוצרת את מבנה התיקיות הסופי.
        """
        if not self.store_data:
            messagebox.showerror("שגיאה", "אין קטגוריות לבניית המבנה.")
            return

        output_dir = filedialog.askdirectory(title="בחר תיקיית פלט (לדוגמה, תיקיית 'app/src/main/assets' בפרויקט)")
        if not output_dir:
            return

        store_path = os.path.join(output_dir, "store")

        if os.path.exists(store_path):
            if messagebox.askyesno("אישור דריסה", f"התיקייה '{store_path}' כבר קיימת.\nהאם לדרוס אותה?"):
                shutil.rmtree(store_path)
            else:
                self.status_label.config(text="הבנייה בוטלה.")
                return
        
        self.status_label.config(text="יוצר מבנה תיקיות, אנא המתן...")
        self.root.update_idletasks()

        try:
            os.makedirs(store_path)
            for folder_name, data in self.store_data.items():
                cat_path = os.path.join(store_path, folder_name)
                os.makedirs(cat_path)
                
                for apk_path in data["apks"]:
                    shutil.copy(apk_path, os.path.join(cat_path, os.path.basename(apk_path)))
            
            messagebox.showinfo("הצלחה", f"מבנה התיקיות 'store' נוצר בהצלחה במיקום:\n{store_path}")
            self.status_label.config(text="הבנייה הושלמה! ניתן לבנות מחדש את פרויקט האנדרואיד.")

        except Exception as e:
            messagebox.showerror("הבנייה נכשלה", f"אירעה שגיאה: {e}")
            self.status_label.config(text="הבנייה נכשלה!")

if __name__ == "__main__":
    # נקודת הכניסה הראשית של הסקריפט
    root = tb.Window(themename="darkly")
    app = StoreBuilderApp(root)
    root.mainloop()